#include <windows.h>
#include <stdio.h> //_snprintf
#include "ConfigForm.h"
#include "resource.h"

bool DialogOpen=false;
HWND MyHwnd;

void SetJuliaConfiguration();

BOOL CALLBACK ConfigProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch(uMsg)
	{
		case WM_INITDIALOG:
			DialogOpen=true;
			MyHwnd=hwndDlg;
			SendMessage(hwndDlg, WM_SETICON, ICON_SMALL, (LPARAM)LoadIcon(GetModuleHandle(NULL), "MAIN_ICON")); //Set the Icon
			break;
		case WM_COMMAND:
			switch(LOWORD(wParam))
			{
				//If OK/APPLY, retrieve all the values, verify them, and return them. OK also closes the dialog.
				case IDOK:
				case CONFIG_APPLY:
				{
					//Retreive doubles
					const int DV[]={CONFIG_X1, CONFIG_X2, CONFIG_Y1, CONFIG_Y2, CONFIG_JuliaConfigReal, CONFIG_JuliaConfigImag}, NumD=sizeof(DV)/sizeof(int); //Doubles to retrieve
					double D[NumD];
					char Buffer[40];
					for(int i=0;i<NumD;i++)
					{
						GetDlgItemText(hwndDlg, DV[i], Buffer, 39);
						D[i]=atof(Buffer);
					}
					if(D[0]>=D[1] || D[2]>=D[3]) //Check boundaries
					{
						MessageBox(hwndDlg, "Lower boundaries cannot exceed upper boundaries.", "Error in configuration", MB_OK);
						return 0;
					}

					//Retreive ints
					const int IV[]={CONFIG_Width, CONFIG_Height, CONFIG_DetailLevel}, NumI=sizeof(IV)/sizeof(int); //Integers to retrieve
					int I[NumI];
					for(i=0;i<NumI;i++)
					{
						BOOL Success;
						I[i]=GetDlgItemInt(hwndDlg, IV[i], &Success, false);
						if(!Success)
						{
							MessageBox(hwndDlg, "One of the integral values is invalid", "Error in configuration", MB_OK);
							return 0;
						}
					}
					if(I[0]<100 || I[1]<100 || I[0]>2000 || I[1]>2000) //Check window dimensions
					{
						MessageBox(hwndDlg, "Window dimensions must be between 100 and 2000.", "Error in configuration", MB_OK);
						return 0;
					}
					if(I[2]<1) //Check detail level
						I[2]=1;

					//Retreive bools
					const int BV[]={CONFIG_Animation, CONFIG_JuliaConfigUsePoint, CONFIG_OGL}, NumB=sizeof(BV)/sizeof(int);
					bool B[NumB];
					for(i=0;i<NumB;i++)
						B[i]=(IsDlgButtonChecked(hwndDlg, BV[i])==BST_CHECKED);

					if(LOWORD(wParam)==IDOK) //Close dialog if "OK" is pressed (apply just applies). This needs to be done before the information is returned because when this window dissapears it might not refresh the window behind it due to message ordering problems.
						DestroyWindow(hwndDlg);
					DialogReturnInformation(D[0], D[1], D[2], D[3], I[0], I[1], I[2], B[0], B[1], D[4], D[5], B[2]);
					break;
				}
				case IDCANCEL: //Just close the dialog on "cancel"
					DestroyWindow(hwndDlg);
					break;
				case CONFIG_JuliaConfigReal:
				case CONFIG_JuliaConfigImag:
				case CONFIG_JuliaConfigUsePoint:
					SetJuliaConfiguration();
					break;
			}
			break;
		case WM_CLOSE:
		case WM_DESTROY:
			DialogOpen=false;
			EndDialog(hwndDlg, IDCANCEL);
	}
	return 0;
}

//All the following functions just set fields in the dialog from parameters. They do not run if the dialog is not open.
void SetJuliaConfiguration()
{
	char Buffer[100];
	GetDlgItemText(MyHwnd, CONFIG_JuliaConfigReal, Buffer, 100);
	double Real=atof(Buffer);
	GetDlgItemText(MyHwnd, CONFIG_JuliaConfigImag, Buffer, 100);
	double Imag=atof(Buffer);
	bool UseC=(IsDlgButtonChecked(MyHwnd, CONFIG_JuliaConfigUsePoint)==BST_CHECKED);

	//Format String
	strcpy(Buffer, "F(z)=z^2");
	if(UseC)
		strcat(Buffer, "+C");
	if(Real!=0)
	{
		if(Real>0)
			strcat(Buffer, "+");
		gcvt(Real, 8, Buffer+strlen(Buffer));
	}
	if(Imag!=0)
	{
		if(Imag>0)
			strcat(Buffer, "+");
		gcvt(Imag, 8, Buffer+strlen(Buffer));
		strcat(Buffer, "i");
	}

	SetDlgItemText(MyHwnd, CONFIG_JuliaConfig, Buffer);
}

void SetConfigBoundaries(double x1, double x2, double y1, double y2) //Also updates zoom
{
	if(!DialogOpen)
		return;

	//Also determine zoom
	double Zoom=300/(x2-x1)-100;
	int ConfigBoxes[]={CONFIG_X1, CONFIG_X2, CONFIG_Y1, CONFIG_Y2, CONFIG_ZoomLevel};
	double ConfigValues[]={x1, x2, y1, y2, Zoom};
	char ConvertText[25];
	for(int i=0;i<5;i++)
	{
		gcvt(ConfigValues[i], 15, ConvertText);
		SetDlgItemText(MyHwnd, ConfigBoxes[i], ConvertText);
	}
}
void SetConfigWindowDimensions(DWORD x, DWORD y)
{
	if(!DialogOpen)
		return;
	SetDlgItemInt(MyHwnd, CONFIG_Width, x, false);
	SetDlgItemInt(MyHwnd, CONFIG_Height, y, false);
}

void SetConfigDetailLevel(DWORD d)
{
	if(!DialogOpen)
		return;
	SetDlgItemInt(MyHwnd, CONFIG_DetailLevel, d, false);
}

void SetConfigPixelInfo(DWORD PixX, DWORD PixY, double SpaceX, double SpaceY, double EscapeV, DWORD Color)
{
	if(!DialogOpen)
		return;

	char Buffer[100];
	_snprintf(Buffer, 100, "%u, %u", PixX, PixY);
	SetDlgItemText(MyHwnd, CONFIG_PixelCoord, Buffer);
	_snprintf(Buffer, 100, "%g, %g", SpaceX, SpaceY);
	SetDlgItemText(MyHwnd, CONFIG_SpaceCoord, Buffer);
	gcvt(EscapeV, 15, Buffer);
	SetDlgItemText(MyHwnd, CONFIG_EscapeValue, Buffer);
	_snprintf(Buffer, 100, "R(%u), G(%u), B(%u) #%06X", Color>>16&0xFF, Color>>8&0xFF, Color&0xFF, Color);
	SetDlgItemText(MyHwnd, CONFIG_Color, Buffer);
}

void SetInitDialogInfo(bool AnimationEnabled, bool UseC, double Real, double Imag, bool UsingOGL)
{
	if(!DialogOpen)
		return;

	CheckDlgButton(MyHwnd, CONFIG_Animation, AnimationEnabled ? BST_CHECKED : BST_UNCHECKED);
	CheckDlgButton(MyHwnd, CONFIG_JuliaConfigUsePoint, UseC ? BST_CHECKED : BST_UNCHECKED);
	CheckRadioButton(MyHwnd, CONFIG_OGL, CONFIG_GDI, UsingOGL ? CONFIG_OGL : CONFIG_GDI);

	char ConvertText[15];
	gcvt(Real, 8, ConvertText);
	SetDlgItemText(MyHwnd, CONFIG_JuliaConfigReal, ConvertText);
	gcvt(Imag, 8, ConvertText);
	SetDlgItemText(MyHwnd, CONFIG_JuliaConfigImag, ConvertText);
}

//Other functions
bool IsDialogOpen() { return DialogOpen; } //Just return if the dialog is open. This can be used by the main window to bypass calculating values to send to the dialog.
void ConfigDisableOGL() //Disable the OpenGL Button
{
		HWND OGLButton=GetDlgItem(MyHwnd, CONFIG_OGL);
		SetWindowLong(OGLButton, GWL_STYLE, GetWindowLong(OGLButton, GWL_STYLE)|WS_DISABLED^WS_VISIBLE);
		RedrawWindow(OGLButton, NULL, NULL, RDW_ERASE|RDW_INVALIDATE);
}