#include "JuliaFractalPixelShader.h"
#include "StdFps.h"

//Configuration
static Fractal F; //The fractal to render -- TO CONFIGURE THE FRACTAL EDIT THIS LINE
static DWORD Width=600, Height=400; //Render surface dimensions
const double MinScale=.000002f; //Fractal stops zooming when scale gets below this number

//Local functions
static LRESULT CALLBACK WndProcNormalRun(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam); //Process windows events

//Main function
const char* NormalRun(HINSTANCE hInst)
{
	//Create the window class
	WNDCLASSEX wc = { sizeof(WNDCLASSEX), NULL, WndProcNormalRun, 0L, 0L, hInst, LoadIcon(hInst, "MAIN_ICON"), LoadCursor(NULL,IDC_ARROW), NULL, NULL, "JuliaFractal", NULL };
	if(RegisterClassEx(&wc)==0)
		return "Cannot register windows class";

	//Determine the size of the actual window that we want based upon client area dimensions
	RECT WindowRect={0, 0, Width, Height};
	AdjustWindowRect(&WindowRect, WS_OVERLAPPEDWINDOW, false);

	//Create the window and engine
	HWND hWnd=CreateWindowEx(NULL, "JuliaFractal", "Julia Fractal", WS_OVERLAPPEDWINDOW, 0, 0, WindowRect.right-WindowRect.left, WindowRect.bottom-WindowRect.top, NULL, NULL, hInst, NULL);
	const char* RenderEngineResult=PrepareRenderEngine(hWnd, Width, Height);
	if(RenderEngineResult)
		return RenderEngineResult;

	//Show the window
	ShowWindow(hWnd, SW_SHOW);

	//Main loop listening for events
	FPS MyFPS;
	MSG msg;
	ZeroMemory(&msg, sizeof(MSG));
	while(true)
	{
		//Check for and handle windows messages
		if(PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE))
		{
			if(GetMessage(&msg, NULL, 0, 0)==0)
				break;
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}

		//Check if a mouse button is being pressed...
		int MouseStates=(GetAsyncKeyState(VK_LBUTTON) ? 1 : 0)|(GetAsyncKeyState(VK_RBUTTON) ? 2 : 0);
		if(!MouseStates)
			continue;

		//Make sure we are in the window boundaries
		POINT CursorPos;
		GetCursorPos(&CursorPos);
		ScreenToClient(hWnd, &CursorPos);
		if(CursorPos.x<0 || CursorPos.y<0 || CursorPos.x>=(int)Width || CursorPos.y>=(int)Height)
			continue;

		//Make sure if zooming in that the maximum zoom level hasn't been reached
		double ScaleX=F.XMax-F.XMin, ScaleY=F.YMax-F.YMin; //Determine the current window dimensions
		if((MouseStates&1) && (ScaleX<=MinScale || ScaleY<=MinScale))
			continue;

		//Zoom in/out around point
		double PointPercentX=(double)CursorPos.x/Width, PointPercentY=(double)CursorPos.y/Height;
		double PointX=F.XMin+PointPercentX*ScaleX, PointY=F.YMin+PointPercentY*ScaleY;
		double ScaleNum=(MouseStates&1 ? 1.0f/1.03 : 1.03);

		F.XMin=PointX-(ScaleX*ScaleNum)*(PointPercentX);
		F.XMax=PointX+(ScaleX*ScaleNum)*(1-PointPercentX);
		F.YMin=PointY-(ScaleY*ScaleNum)*(PointPercentY);
		F.YMax=PointY+(ScaleY*ScaleNum)*(1-PointPercentY);

		DrawFractal(F);

		//Update the window title with the current FPS
		MyFPS.AddFrame();
		char WindowTitle[100]="Julia Fractal @ ";
		strcat(WindowTitle, MyFPS.GetFpsString());
		SetWindowText(hWnd, WindowTitle);
	}

	return NULL;
}

LRESULT CALLBACK WndProcNormalRun(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam) //Process windows events
{
	switch(msg)
	{
		case WM_SIZE:
		{
			//Store the new client dimensions
			Width=LOWORD(lParam);
			Height=HIWORD(lParam);

			//Release and Create the D3D Interface
			PrepareRenderEngine(hWnd, Width, Height);
			DrawFractal(F);
			break;
		}
		case WM_CHAR:
			if(wParam=='=' || wParam=='-') //Detail level up
			{
				if(wParam=='=')
					F.MaxIterations*=2;
				else
					F.MaxIterations/=2;

				DrawFractal(F);
			}
			else //Assumes a direction key was pressed
			{
				//Move in given direction
				double ScaleX=F.XMax-F.XMin, ScaleY=F.YMax-F.YMin; //Determine the current window dimensions

				//Determine the center of the window
				double PointX=.5*ScaleX+F.XMin;
				double PointY=F.YMax-.5*ScaleY;

				if(wParam=='w')
					PointY+=ScaleY/-4;
				else if(wParam=='a')
					PointX+=ScaleX/-4;
				else if(wParam=='s')
					PointY+=ScaleY/4;
				else if(wParam=='d')
					PointX+=ScaleX/4;
				else //If not w/a/s/d, do nothing
					break;

				F.XMin=PointX-ScaleX/2;
				F.XMax=PointX+ScaleX/2;
				F.YMin=PointY-ScaleY/2;
				F.YMax=PointY+ScaleY/2;

				DrawFractal(F);
			}
			break;
		case WM_PAINT:
			//Refresh window
			DrawFractal(F);
			break;
		case WM_DESTROY:
			//End the program
			PostQuitMessage(0);
			return 0;
	}
	return DefWindowProc(hWnd, msg, wParam, lParam);
}