#include <stdio.h>
#include <io.h>
#include <fcntl.h>
#include <string.h>
#include "Settings.h"

#define DEBUGGING 0

//Settings
const char *Mode, *IsFullScreen, *Width, *Height, *ShrinkConstant, *TextureFilter, *DetailLevel, *ViewAngle, *XRot, *YRot, *ZRot, *IsRotating, *IsPlaying, *IsImmediate;
const char* SettingNames[]={"Mode", "IsFullScreen", "Width", "Height", "ShrinkConstant", "TextureFilter", "DetailLevel", "ViewAngle", "XRot", "YRot", "ZRot", "IsRotating", "IsPlaying", "IsImmediate"};
const char** SettingPointers[]={&Mode, &IsFullScreen, &Width, &Height, &ShrinkConstant, &TextureFilter, &DetailLevel, &ViewAngle, &XRot, &YRot, &ZRot, &IsRotating, &IsPlaying, &IsImmediate};
const int NumSettings=sizeof(SettingNames)/sizeof(const char*);

void ParseSettings(const char* FileName) //Retreive the settings from the provided file
{
	//NULL out all settings
	for(int n=0;n<NumSettings;n++)
		*SettingPointers[n]=NULL;

	//Process settings from the configuration file
	int SettingsFile=open(FileName, O_RDONLY);
	char *SettingsContent=NULL;
	if(SettingsFile!=-1)
	{
		//Read in the whole file
		int FileLen=lseek(SettingsFile, 0, SEEK_END);
		lseek(SettingsFile, 0, SEEK_SET);
		SettingsContent=new char[FileLen+1];
		int AmountRead=read(SettingsFile, SettingsContent, FileLen);
		SettingsContent[AmountRead]=0;
		close(SettingsFile);

		//Process the settings with strtok
		for(char *CurString=strtok(SettingsContent, "\n");CurString;CurString=strtok(NULL, "\n"))
		{
			int StartOfName, EndOfName, StartOfValue, EndOfValue;

			//Find the name string
			for(StartOfName=0;CurString[StartOfName]==' ' || CurString[StartOfName]=='\t';StartOfName++); //Skip past whitespace
			for(EndOfName=StartOfName;CurString[EndOfName]!='#' && CurString[EndOfName]!='=' && CurString[EndOfName]!='\0';EndOfName++); //Find end of line or value delimiter
			if(CurString[EndOfName]!='=') //Bypass comment lines and EOF
				continue;
			StartOfValue=EndOfName+1;
			for(--EndOfName;CurString[EndOfName]==' ' || CurString[EndOfName]=='\t';EndOfName--); //Remove whitespace from the end of the name
			CurString[EndOfName+1]=0;

			//Find the setting if it exists
			int SettingNum;
			for(SettingNum=0;SettingNum<NumSettings;SettingNum++)
				if(stricmp(SettingNames[SettingNum], CurString+StartOfName)==0)
					break;
			#if !DEBUGGING
				if(SettingNum==NumSettings) //Skip finding and setting the value if if doesn't exist
					continue;
			#endif

			//Find the value string
			for(;CurString[StartOfValue]==' ' || CurString[StartOfValue]=='\t';StartOfValue++); //Skip past whitespace
			for(EndOfValue=StartOfValue;CurString[EndOfValue]!='#' && CurString[EndOfValue]!='\0';EndOfValue++); //Find the end of line
			for(--EndOfValue;CurString[EndOfValue]==' ' || CurString[EndOfValue]=='\t';EndOfValue--); //Remove whitespace from the end of the value
			CurString[EndOfValue+1]=0;

			//Set the value
			#if DEBUGGING
				printf("'%s'='%s'\n", CurString+StartOfName, CurString+StartOfValue);
				if(SettingNum!=NumSettings) //Only need to check the next line if in debugging mode
			#endif
			*SettingPointers[SettingNum]=CurString+StartOfValue;
		}
	}

	//Copy the settings strings
	for(int i=0;i<NumSettings;i++)
	{
		const char** Setting=SettingPointers[i];
		if(!*Setting) //Nothing to copy for blank settings
			continue;
		char* SettingCopy=new char[strlen(*Setting)+1];
		strcpy(SettingCopy, *Setting);
		*Setting=SettingCopy;
	}

	//If a file was opened, get rid of its information
	if(SettingsFile!=-1)
		delete[] SettingsContent;
}

const char* GetSetting(const char* Name, const char* Default) //Retreive the string value for the setting
{
	for(int i=0;i<NumSettings;i++) //Check all names for the requested one
		if(stricmp(Name, SettingNames[i])==0)
			return (*SettingPointers[i] ? *SettingPointers[i] : Default); //If value was not found in the config file, return the default
	return NULL; //If name is not found, return NULL
}

void ReleaseSettings() //Release settings retreived from the file
{
	//Erase and NULL out all settings
	for(int i=0;i<NumSettings;i++)
	{
		if(*SettingPointers[i])
			delete[] (char*)*SettingPointers[i];
		*SettingPointers[i]=NULL;
	}
}